"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const Context = __importStar(require("../p2p/Context"));
const zlib_1 = __importDefault(require("zlib"));
const trie_prefix_tree_1 = __importDefault(require("trie-prefix-tree"));
const debugMiddleware_1 = require("../network/debugMiddleware");
const nestedCounters_1 = require("../utils/nestedCounters");
const logger_1 = require("../logger");
const ProblemNodeHandler = __importStar(require("../p2p/ProblemNodeHandler"));
const NodeList_1 = require("../p2p/NodeList");
const tar = require('tar-fs');
const fs = require('fs');
class Debug {
    constructor(baseDir, network) {
        this.baseDir = baseDir;
        this.network = network;
        this.archiveName = `debug-${network.ipInfo.externalIp}-${network.ipInfo.externalPort}.tar.gz`;
        this.files = {};
        this._registerRoutes();
    }
    addToArchive(src, dest) {
        if (path_1.default.isAbsolute(dest))
            throw new Error('"dest" must be a relative path.');
        src = path_1.default.isAbsolute(src) ? src : path_1.default.resolve(path_1.default.join(this.baseDir, src));
        this.files[src] = dest;
    }
    createArchiveStream() {
        const cwd = process.cwd();
        const filesRel = {};
        for (const src in this.files) {
            const srcRel = path_1.default.relative(cwd, src).replace(/\\/g, '/');
            const dest = this.files[src];
            filesRel[srcRel] = dest;
        }
        const entries = Object.keys(filesRel);
        const trie = (0, trie_prefix_tree_1.default)(entries);
        const pack = tar.pack(cwd, {
            entries,
            map: function (header) {
                let entry = header.name.replace(/\\/g, '/');
                // Find the closest entry for this item
                while (!trie.isPrefix(entry) && entry.length > 0) {
                    entry = entry.slice(0, -1);
                }
                if (entry.length === 0) {
                    if (logger_1.logFlags.error) {
                        nestedCounters_1.nestedCountersInstance.countEvent('debug', 'error: No valid entry found for header');
                        this.main.logger.error('debug: No valid entry found for header:', header.name);
                    }
                    return header;
                }
                // Remove srcRel from header.name
                header.name = path_1.default.relative(entry, header.name);
                // Prefix dest to whatever remains
                const dest = filesRel[entry];
                if (!dest) {
                    if (logger_1.logFlags.error) {
                        nestedCounters_1.nestedCountersInstance.countEvent('debug', 'error: Destination not found for entry');
                        this.main.logger.error('debug: Destination not found for entry:', entry);
                    }
                    return header;
                }
                header.name = path_1.default.normalize(path_1.default.join(dest, header.name));
                return header;
            },
        });
        return pack;
    }
    _registerRoutes() {
        this.network.registerExternalGet('debug', debugMiddleware_1.isDebugModeMiddlewareMedium, (req, res) => {
            const archive = this.createArchiveStream();
            const gzip = zlib_1.default.createGzip();
            res.set('content-disposition', `attachment; filename="${this.archiveName}"`);
            res.set('content-type', 'application/gzip');
            archive.pipe(gzip).pipe(res);
        });
        this.network.registerExternalGet('debug-logfile', debugMiddleware_1.isDebugModeMiddlewareMedium, (req, res) => {
            const requestedFile = req.query.file;
            if (typeof requestedFile !== 'string' || !requestedFile) {
                res.json({ success: false, error: 'Invalid file parameter' });
                return;
            }
            const normalizedFile = path_1.default.normalize(requestedFile).replace(/^(\.\.[/\\])+/, '');
            const logsAbsolutePath = Object.keys(this.files).find((key) => this.files[key] === './logs');
            if (!logsAbsolutePath) {
                res.json({ success: false, error: 'Logs directory not found' });
                return;
            }
            const filePath = path_1.default.join(logsAbsolutePath, normalizedFile);
            if (!filePath.startsWith(logsAbsolutePath)) {
                res.json({ success: false, error: 'File not found' });
                return;
            }
            res.set('Content-Disposition', `attachment; filename="${path_1.default.basename(normalizedFile)}"`);
            res.set('Content-Type', 'text/plain');
            const fileStream = fs.createReadStream(filePath);
            fileStream.on('error', (error) => {
                res.json({ success: false, error: 'Error reading the file' });
                return;
            });
            fileStream.pipe(res);
        });
        this.network.registerExternalGet('debug-network-delay', debugMiddleware_1.isDebugModeMiddleware, (req, res) => {
            try {
                const delay = req.query.delay && typeof req.query.delay === 'string' ? parseInt(req.query.delay) : 120 * 1000;
                this.network.setDebugNetworkDelay(delay);
            }
            catch (e) {
                res.json({ success: false, error: e.message });
                return;
            }
            res.json({ success: true });
            return;
        });
        this.network.registerExternalGet('debug-forcedExpiration', debugMiddleware_1.isDebugModeMiddleware, (req, res) => {
            try {
                const forcedExpiration = req.query.forcedExpiration && typeof req.query.forcedExpiration === 'string'
                    ? req.query.forcedExpiration === 'true'
                    : false;
                Context.config.debug.forcedExpiration = forcedExpiration;
                nestedCounters_1.nestedCountersInstance.countEvent('debug', `forcedExpiration set to ${forcedExpiration}`);
            }
            catch (e) {
                res.json({ success: false, error: e.message });
                return;
            }
            res.json({ success: true });
            return;
        });
        this.network.registerExternalGet('debug_problemNodeTrackerDump', debugMiddleware_1.isDebugModeMiddleware, (_, res) => {
            try {
                const dump = {};
                let lastCycle = Context.p2p.state.getLastCycle();
                // Collect data for all nodes that have any refute history
                for (const [nodeId, node] of NodeList_1.nodes) {
                    if (node.refuteCycles?.length > 0) {
                        const refuteCycles = node.refuteCycles;
                        dump[nodeId] = {
                            refuteCycles,
                            stats: {
                                refutePercentage: ProblemNodeHandler.getRefutePercentage(node.refuteCycles, lastCycle.counter),
                                consecutiveRefutes: ProblemNodeHandler.getConsecutiveRefutes(refuteCycles, lastCycle.counter),
                                isProblematic: ProblemNodeHandler.isNodeProblematic(node, lastCycle.counter),
                            },
                        };
                    }
                }
                res.json({ success: true, data: { cycle: lastCycle.counter, nodeHistories: dump } });
            }
            catch (e) {
                res.json({ success: false, error: e.message });
            }
        });
    }
}
exports.default = Debug;
//# sourceMappingURL=debug.js.map